from gettext import gettext as _
import importlib
import re
import sys

from . import providers
from .exceptions import SocialError


# Helps to ensure that providers are safe module names.
provider_pattern = re.compile(r'^[a-z][a-z_]+[a-z]$')


# Only define error messages once.
exist_msg = _("The popular provider %s does not exist.")


class Manager(object):
    """Manages interactions with providers.

    This class is the developer's simplest interface with the social
    providers. It condenses things like credentials and calling
    providers by name.
    """

    def __init__(self, config):
        """Sets up the manager with configuration details for providers.

        The configuration should be a dict that looks like:
            {
                'facebook': {
                    'keys': 'that',
                    'are': 'used',
                    'by': 'facebook',
                },
                'google': {...},
            }

        This configuration also determines which providers will be made
        available through the manager, otherwise raising exceptions.

        Args:
            config: a dict representing all configured social providers.

        Raises:
            SocialError: The popular provider "%s" does not exist.
            ValueError: The popular configuration must be a dict.
        """
        if not isinstance(config, dict):
            raise ValueError(_(
                "The popular configuration must be a dict."
            ))
        self.providers = dict()
        for name in config:
            if not provider_pattern.match(name):
                raise SocialError(exist_msg % name)
            try:
                module = importlib.import_module(
                    ".%s" % name, providers.__name__
                )
            except ImportError:
                raise SocialError(exist_msg % name)
            try:
                provider = module.provider
            except AttributeError:
                raise SocialError(exist_msg % name)
            self.providers[name] = provider(config[name])

    def provider(self, name):
        """Returns the provider of choice.

        Args:
            name: a string name identifying the social provider.

        Returns:
            A subclass of popular.providers.base.Provider.

        Raises:
            SocialError: The popular provider "%s" does not exist.
        """
        try:
            return self.providers[name]
        except KeyError:
            raise SocialError(exist_msg % name)



import pytest


from .facebook import FacebookProvider
from .github import GithubProvider
from .google import GoogleProvider


def test_facebook_provider_auth_url_success():
    provider = FacebookProvider({
        'client_id': 'moose',
        'client_secret': 'moose',
        'redirect_uri': 'https://moose.com/callback',
    })
    url = provider.get_auth_url(state='moose')


def test_github_provider_auth_url_success():
    provider = GithubProvider({
        'client_id': 'moose',
        'client_secret': 'moose',
        'redirect_uri': 'https://moose.com/callback',
    })
    url = provider.get_auth_url(state='moose')


def test_google_provider_auth_url_success():
    provider = GoogleProvider({
        'client_id': 'moose',
        'client_secret': 'moose',
        'redirect_uri': 'https://moose.com/callback',
    })
    url = provider.get_auth_url(state='moose')



from gettext import gettext as _
import requests

from .base import Provider
from ..exceptions import SocialError, SocialProviderError
from ..users import User


class GoogleProvider(Provider):
    """Provider for github.com authentication."""

    CONFIG_KEYS = [
        'client_id',
        'client_secret',
        'redirect_uri',
    ]

    def get_auth_url(self, state):
        """Generates the url for the user to grant permission on.

        Args:
            state: a string of random characters to help prevent CSRF
                attacks.

        Returns:
            A string URL.
        """
        url = 'https://accounts.google.com/o/oauth2/v2/auth'
        return self.serialize_url(url=url, params=dict(
            client_id=self.config['client_id'],
            redirect_uri=self.config['redirect_uri'],
            scope=' '.join([
                'https://www.googleapis.com/auth/userinfo.email',
                'https://www.googleapis.com/auth/userinfo.profile',
            ]),
            state=state,
            access_type='online',
            response_type='code',
        ))

    def get_user(self, uri, state):
        """Takes the response URI and retrieves a user from it.

        Args:
            uri: a string uri that the service sent the user to,
                including all query paramters attached.
            state: a string that was provided for this exact request
                when the user was first redirected.

        Returns:
            A popular.users.User instance.

        Raises:
            The state parameter is invalid.
        """
        # See if the uri has what we expect.
        uri_params = self.parse_uri(uri, required=['code', 'state'])
        if state:
            if uri_params['state'] != state:
                raise SocialError(_("The state parameter is invalid."))

        # Get the access token from the API.
        url = 'https://www.googleapis.com/oauth2/v4/token'
        data = dict(
            client_id=self.config['client_id'],
            client_secret=self.config['client_secret'],
            redirect_uri=self.config['redirect_uri'],
            code=uri_params['code'],
            grant_type='authorization_code',
        )
        r = requests.post(url, data=data)
        access_token = self.response_to_dict(r)['access_token']

        # Grab the user from the API.
        url = 'https://www.googleapis.com/plus/v1/people/me'
        headers = {
            'Accept': 'application/json',
            'Authorization': 'Bearer %s' % access_token,
        }
        r = requests.get(url, headers=headers, params={'prettyPrint': 'false'})
        raw = self.response_to_dict(r)
        user = User()
        user.set_raw(raw)
        user.map(
            id=raw['id'],
            name=raw['displayName'],
            email=raw['emails'][0]['value'],
            avatar=raw['image']['url'],
        )
        return user

    def response_to_dict(self, response):
        """Helper to gracefully return error messages from API."""
        output = response.json()
        if response.status_code != 200:
            raise SocialProviderError(output['message'])
        if 'error' in output:
            raise SocialProviderError(output['error'])
        return output


# Make a consistent reference for the Manager to use.
provider = GoogleProvider



import pytest

from .exceptions import SocialError
from . import Popular as Manager


def test_manager_empty_success():
    Manager({})

def test_manager_github_success():
    Manager({
        'github': {
            'client_id': 'moose',
            'client_secret': 'moose',
            'redirect_uri': 'moose',
        },
    })

def test_manager_github_failure():
    with pytest.raises(SocialError) as err:
        Manager({
            'github': {
                'client_id': 'moose',
                'client_secret': 'moose',
                '0redirect_uri': 'moose',
            },
        })
    assert str(err.value).startswith(
        'The github provider requires the following keys: '
    )

def test_manager_type_failure():
    with pytest.raises(ValueError) as err:
        Manager('moose')
    assert str(err.value) == "The popular configuration must be a dict."

def test_manager_nonexistant_failure():
    with pytest.raises(SocialError) as err:
        Manager({'base': {}})
    assert str(err.value) == 'The popular provider base does not exist.'
    with pytest.raises(SocialError) as err:
        Manager({'moose': {}})
    assert str(err.value) == 'The popular provider moose does not exist.'



"""Popular

This package is a minimalist take on social authentication. It focuses
on the OAuth process as two main steps.

    1) Request permission from a user on a service.
        - IN: service name, application secret, etc
        - OUT: string URL
    2) Request auth tokens from a service on behalf of a user.
        - IN: string URL
        - OUT: tokens, user info, etc

This package stays out of the implementation and focuses on the
interaction with the social auth provider.
"""

__license__ = 'MIT License'
__version__ = '0.1.4'


from .manager import Manager as Popular



from gettext import gettext as _

from ..exceptions import SocialError
from ..utils import dict_to_query_string, uri_to_query_string_params


class Provider(object):
    """Base class for a social provider.

    Social providers are services like Facebook and Google who manage
    accounts for users. A Provider subclass will fill in the underlying
    functionality required to interact with those services.
    """

    # Configuration keys are each checked for on instantiation to let
    # the developer know what exactly is required.
    CONFIG_KEYS = []

    def __init__(self, config):
        """Does basic validation for the provider.

        This doesn't reach out to the actual service to validate
        anything, but rather it makes sure the proper ingredients and
        credentials are present.

        The required configuration details will change from provider to
        provider.

        Args:
            config: a dict containing credentials for a service provider
                application.

        Raises:
            SocialError: The %s provider requires the following
                keys: %s.
            TypeError: The "%s" must be a string.
            ValueError: The popular configuration for %s must be a
                dict.
        """
        assert self.CONFIG_KEYS
        self.name = self.__class__.__module__.split('.')[-1]
        if not isinstance(config, dict):
            raise ValueError(_(
                "The popular configuration for %s must be a dict."
            ) % self.name)
        if set(config.keys()) != set(self.CONFIG_KEYS):
            raise SocialError(
                _("The %s provider requires the following keys: %s.") % (
                    self.name,
                    ', '.join(self.CONFIG_KEYS),
                )
            )
        for name in config:
            if not isinstance(config[name], str):
                raise TypeError(_("The \"%s\" must be a string.") % name)
        self.config = config

    def get_auth_url(self, state):
        """Generates the url for the user to grant permission on.

        Args:
            state: a string of random characters to help prevent CSRF
                attacks.
            state: a string that was provided for this exact request
                when the user was first redirected.

        Returns:
            A string URL.
        """
        raise NotImplementedError()

    def get_user(self, uri, state):
        """Takes the response URI and retrieves a user from it.

        Args:
            uri: a string uri that the service sent the user to,
                including all query paramters attached.
            state: a string that was provided for this exact request
                when the user was first redirected.

        Returns:
            A popular.users.User instance.
        """
        raise NotImplementedError()

    def parse_uri(self, uri, required=None):
        """Parses the uri from the vendor.

        This will make sure the required parameters are present.

        Args:
            uri: a string URI.
            required: a list of parameter names.

        Returns:
            A dict of parsed query parameters.

        Raises:
            SocialError: Required query parameter \"%s\" is not
                present.
        """
        params = uri_to_query_string_params(uri)
        for param in required:
            if param not in params:
                raise SocialError(
                    _("Required query parameter \"%s\" is not present.") % param
                )
        return params

    def serialize_url(self, url, params):
        """Helper to add a query string to a url."""
        return '%s?%s' % (url, dict_to_query_string(params),)



from urllib.parse import quote_plus, unquote


def dict_to_query_string(d):
    """Helper to serialize URL-safe parameters into a string."""
    pairs = list()
    for name in d:
        pairs.append('='.join([
            quote_plus(name),
            quote_plus(d[name]),
        ]))
    return '&'.join(pairs)


def uri_to_query_string_params(uri):
    """Helper to deserialize query string into a dictionary.

    Args:
        uri: expects a uri that starts with https?:// or /.
    """
    output = dict()
    if '?' not in uri:
        return output
    qs = uri[uri.find('?')+1:].split('#')[0]
    pairs = qs.split('&')
    for pair in pairs:
        if '=' not in pair:
            output[unquote(pair)] = True
            continue
        elems = pair.split('=')
        output[unquote(elems[0])] = unquote(elems[1])
    return output



from gettext import gettext as _
import requests

from .base import Provider
from ..exceptions import SocialError, SocialProviderError
from ..users import User


class GithubProvider(Provider):
    """Provider for github.com authentication."""

    CONFIG_KEYS = [
        'client_id',
        'client_secret',
        'redirect_uri',
    ]

    def get_auth_url(self, state):
        """Generates the url for the user to grant permission on.

        Args:
            state: a string of random characters to help prevent CSRF
                attacks.

        Returns:
            A string URL.
        """
        url = 'http://github.com/login/oauth/authorize'
        return self.serialize_url(url=url, params=dict(
            client_id=self.config['client_id'],
            redirect_uri=self.config['redirect_uri'],
            scope=' '.join(['user:email']),
            state=state,
            allow_signup='true',
        ))

    def get_user(self, uri, state):
        """Takes the response URI and retrieves a user from it.

        Args:
            uri: a string uri that the service sent the user to,
                including all query paramters attached.
            state: a string that was provided for this exact request
                when the user was first redirected.

        Returns:
            A popular.users.User instance.

        Raises:
            The state parameter is invalid.
        """
        # See if the uri has what we expect.
        uri_params = self.parse_uri(uri, required=['code', 'state'])
        if uri_params['state'] != state:
            raise SocialError(_("The state parameter is invalid."))

        # Get the access token from the API.
        url = 'https://github.com/login/oauth/access_token'
        headers = {'Accept': 'application/json'}
        data = dict(
            client_id=self.config['client_id'],
            client_secret=self.config['client_secret'],
            redirect_uri=self.config['redirect_uri'],
            code=uri_params['code'],
            state=state,
        )
        r = requests.post(url, headers=headers, data=data)
        access_token = self.response_to_dict(r)['access_token']

        # Grab the user from the API.
        url = 'https://api.github.com/user'
        headers = {
            'Accept': 'application/json',
            'Authorization': 'token %s' % access_token,
        }
        r = requests.get(url, headers=headers)
        raw = self.response_to_dict(r)
        user = User()
        user.set_raw(raw)
        user.map(
            id=raw['id'],
            name=raw['name'],
            nickname=raw['login'],
            avatar=raw.get('avatar_url', None),
        )

        # Grab the email from the API.
        url = 'https://api.github.com/user/emails'
        headers = {
            'Accept': 'application/json',
            'Authorization': 'token %s' % access_token,
        }
        r = requests.get(url, headers=headers)
        raw = self.response_to_dict(r)
        for email in raw:
            user.map(email=email['email'])
            if email['primary'] == True:
                break
        return user

    def response_to_dict(self, response):
        """Helper to gracefully return error messages from API."""
        output = response.json()
        if response.status_code != 200:
            raise SocialProviderError(output['message'])
        if 'error' in output:
            raise SocialProviderError(output['error'])
        return output


# Make a consistent reference for the Manager to use.
provider = GithubProvider



import os
import os.path
import re
from setuptools import setup


def get_info(var, pkg):
    """Get version from the package."""
    with open(os.path.join(pkg,'__init__.py')) as f:
        content = f.read()
    return re.search(var + r'\s*=\s*["\'](.+?)["\']', content).group(1)


def get_packages(package):
    """
    Taken from https://github.com/tomchristie/django-rest-framework/blob/master/setup.py
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


NAME = 'popular'
VERSION = get_info('__version__', NAME)
LICENSE = get_info('__license__', NAME)


setup(
    name=NAME,
    description="Minimalist social auth package.",
    url="https://github.com/ryannjohnson/popular-python",
    author="Ryan N Johnson",
    author_email="ryan@ryannjohnson.com",
    license=LICENSE,
    version=VERSION,
    packages=get_packages(NAME),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    install_requires=[
        'requests>=2.14.2',
    ],
)





class User(object):
    """Information about an authenticated user.

    Providers return different forms of information. This container is
    meant to provide a common interface with that information across all
    vendors.
    """

    ATTRIBUTES = [
        'id',
        'name',
        'nickname',
        'email',
        'avatar',
    ]

    def __getattr__(self, key):
        if key in self.ATTRIBUTES:
            return None
        raise AttributeError("The attribute \"%s\" does not exist." % key)

    def map(self, **kwargs):
        """Turns a dictionary into the properties of this user.

        Args:
            **kwargs: a dictionary of keys and values to merge into this
                instance.

        Raises:
            KeyError: Cannot map attribute "%s" to user.
        """
        for name in kwargs:
            if name not in self.ATTRIBUTES:
                raise KeyError("Cannot map attribute \"%s\" to user." % name)
            setattr(self, name, kwargs[name])

    def set_raw(self, user):
        """Saves the original data here."""
        self.user = user

    def to_dict(self):
        output = dict()
        for a in self.ATTRIBUTES:
            output[a] = getattr(self, a, None)
        return output
